#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   download.py
@Time    :   2020/08/18
@Author  :   Yaronzz
@Version :   1.0
@Contact :   yaronhuang@foxmail.com
@Desc    :   
'''
import os

import aigpy.m3u8Helper as m3u8Helper
from aigpy.tagHelper import TagTool
from aigpy.netHelper import downloadFileRetErr, downloadFile
from aigpy.stringHelper import isNull, getSubOnlyEnd
from aigpy.pathHelper import replaceLimitChar, getFileName, remove
from aigpy.fileHelper import getFileContent

from tidal_dl.tidal import TidalAPI
from tidal_dl.enum import Type, AudioQuality, VideoQuality
from tidal_dl.printf import Printf
from tidal_dl.decryption import decrypt_security_token
from tidal_dl.decryption import decrypt_file

API = TidalAPI()

def __loadAPI__(user):
    API.key.accessToken = user.assesstoken
    API.key.userId = user.userid
    API.key.countryCode = user.countryCode
    API.key.sessionId = user.sessionid1


def __loadVideoAPI__(user):
    API.key.accessToken = user.assesstoken
    API.key.userId = user.userid
    API.key.countryCode = user.countryCode
    API.key.sessionId = user.sessionid2 if not isNull(user.sessionid2) else user.sessionid1



def __getIndexStr__(index):
    pre = "0"
    if index < 10:
        return pre+str(index)
    if index < 99:
        return str(index)
    return str(index)

def __getExtension__(url):
    if '.flac' in url:
        return '.flac'
    if '.mp4' in url:
        return '.mp4'
    return '.m4a'

def __getArtists__(array):
    ret = []
    for item in array:
        ret.append(item.name)
    return ret



def __setMetaData__(track, album, filepath):
    obj = TagTool(filepath)
    obj.album = track.album.title
    obj.title = track.title
    obj.artist = __getArtists__(track.artists)
    obj.copyright = track.copyRight
    obj.tracknumber = track.trackNumber
    obj.discnumber = track.volumeNumber
    obj.isrc = track.isrc
    obj.albumartist = __getArtists__(album.artists)
    obj.date = album.releaseDate
    obj.totaldisc = album.numberOfVolumes
    if obj.totaldisc <= 1:
        obj.totaltrack = album.numberOfTracks
    coverpath = API.getCoverUrl(album.cover, "1280", "1280")
    obj.save(coverpath)
    return

def __convertToM4a__(filepath, codec):
    if 'ac4' in codec or 'mha1' in codec:
        return filepath
    if '.mp4' not in filepath:
        return filepath
    newpath = filepath.replace('.mp4', '.m4a')
    remove(newpath)
    os.rename(filepath, newpath)
    return newpath

def __getAlbumPath__(conf, album):
    # outputdir/Album/artist/
    artist = replaceLimitChar(album.artists[0].name, '-')
    base = conf.downloadPath + '/Album/' + artist + '/'
    
    #album folder pre: [ME][ID]
    flag = API.getFlag(album, Type.Album, True, "")
    if conf.audioQuality != AudioQuality.Master:
        flag.replace('M',"")
    if not isNull(flag):
        flag = "[" + flag + "] "
    
    sid = "[" + str(album.id) + "] " if conf.addAlbumIDBeforeFolder else ""

    #album and addyear
    albumname = replaceLimitChar(album.title, '-')
    year = ""
    if conf.addYear:
        year = "[" + getSubOnlyEnd(album.releaseDate, '-') + "] "
    return base + flag + sid + year + albumname + '/'

def __getPlaylistPath__(conf, playlist):
    pass

def __getTrackPath__(conf, track, stream, album=None, playlist=None):
    if album is not None:
        base = __getAlbumPath__(conf, album)
    if playlist is not None:
        base = __getPlaylistPath__(conf, playlist)
    #CD
    if album.numberOfVolumes > 1:
        base += 'CD' + str(track.volumeNumber) + '/'
    # hyphen
    hyphen = ' - ' if conf.addHyphen else ' '
    # get number
    number = ''
    if conf.useTrackNumber:
        number = __getIndexStr__(track.trackNumber) + hyphen
    # get artist
    artist = ''
    if conf.artistBeforeTitle:
        artist = replaceLimitChar(track.artists[0].name, '-') + hyphen
    # get explicit
    explicit = "(Explicit)" if conf.addExplicitTag and track.explicit else ''
    # title
    title = track.title
    if not isNull(track.version):
        title += ' - ' + track.version
    title = replaceLimitChar(title, '-')
    # extension
    extension = __getExtension__(stream.url)
    return base + number + artist + title + explicit + extension


def __getVideoPath__(conf, video, album=None, playlist=None):
    if album is not None:
        base = __getAlbumPath__(conf, album)
    elif playlist is not None:
        base = __getPlaylistPath__(conf, playlist)
    else:
        base = conf.downloadPath + '/Video/'
     
    # hyphen
    hyphen = ' - ' if conf.addHyphen else ' '
    # get number
    number = ''
    if conf.useTrackNumber:
        number = __getIndexStr__(video.trackNumber) + hyphen
    # get artist
    artist = ''
    if conf.artistBeforeTitle:
        artist = replaceLimitChar(video.artists[0].name, '-') + hyphen
    # get explicit
    explicit = "(Explicit)" if conf.addExplicitTag and video.explicit else ''
    # title
    title = replaceLimitChar(video.title, '-')
    # extension
    extension = ".mp4"
    return base + number + artist + title + explicit + extension
    




def __downloadVideo__(conf, video, album=None, playlist=None):
    msg, stream = API.getVideoStreamUrl(video.id, conf.videoQuality)
    if not isNull(msg):
        Printf.err(video.title + "." + msg)
        return
    path = __getVideoPath__(conf, video, album, playlist)
    if m3u8Helper.download(stream.m3u8Url, path):
        Printf.success(getFileName(path))
    else:
        Printf.err("\nDownload failed!" + getFileName(path) )

def __downloadTrack__(conf, track, album=None, playlist=None):
    try:
        msg, stream = API.getStreamUrl(track.id, conf.audioQuality)
        if not isNull(msg):
            Printf.err(track.title + "." + msg)
            return
        path = __getTrackPath__(conf, track, stream, album, playlist)

        # Printf.info("Download \"" + track.title + "\" Codec: " + stream.codec)
        check, err = downloadFileRetErr(stream.url, path + '.part', showprogress=True, stimeout=20)
        if not check:
            Printf.err("\n Download failed!" + getFileName(path) )
            return
        # encrypted -> decrypt and remove encrypted file
        if isNull(stream.encryptionKey):
            os.replace(path + '.part', path)
        else:
            key, nonce = decrypt_security_token(stream.encryptionKey)
            decrypt_file(path + '.part', path, key, nonce)
            os.remove(path +'.part')

        path = __convertToM4a__(path, stream.codec)
        __setMetaData__(track, album, path)
        Printf.success(getFileName(path))
    except:
        Printf.err("\n Download failed!" + track.title)

def __downloadCover__(conf, album):
    if album == None:
        return
    path = __getAlbumPath__(conf, album) + '/cover.jpg'
    url = API.getCoverUrl(album.cover, "1280", "1280")
    downloadFile(url, path)







def __album__(conf, obj):
    Printf.album(obj)
    msg, tracks, videos = API.getItems(obj.id, Type.Album)
    if not isNull(msg):
        Printf.err(msg)
        return
    if conf.saveCovers:
        __downloadCover__(conf, obj)
    for item in tracks:
        __downloadTrack__(conf, item, obj)
    for item in videos:
        __downloadVideo__(conf, item, obj)

def __track__(conf, obj):
    Printf.track(obj)
    msg, album = API.getAlbum(obj.album.id)
    if conf.saveCovers:
        __downloadCover__(conf, album)
    __downloadTrack__(conf, obj, album)

def __video__(conf, obj):
    Printf.video(obj)
    __downloadVideo__(conf, obj, obj.album)

def __artist__(conf, obj):
    Printf.artist(obj)
    msg, albums = API.getArtistAlbums(obj.id, conf.includeEP)
    if not isNull(msg):
        Printf.err(msg)
        return
    for item in albums:
        __album__(conf, item)

def __playlist__(conf, obj):
    Printf.playlist(obj)
    msg, tracks, videos = API.getItems(obj.uuid, Type.Playlist)
    if not isNull(msg):
        Printf.err(msg)
        return

    for item in tracks:
        mag, album = API.getAlbum(item.album.id)
        __downloadTrack__(conf, item, album)
    for item in videos:
        __downloadVideo__(conf, item, None)

def __file__(user, conf, string):
    txt = getFileContent(string)
    if isNull(txt):
        Printf.err("Nothing can read!")
        return
    array = txt.split('\n')
    for item in array:
        if isNull(item):
            continue
        if item[0] == '#':
            continue
        if item[0] == '[':
            continue
        start(user, conf, item)

def start(user, conf, string):
    __loadAPI__(user)

    if os.path.exists(string):
        __file__(user, conf, string)
        return

    msg, etype, obj = API.getByString(string)
    if etype == Type.Null or not isNull(msg):
        Printf.err(msg)
        return

    if etype == Type.Album:
        __album__(conf, obj)
    if etype == Type.Track:
        __track__(conf, obj)
    if etype == Type.Video:
        __loadVideoAPI__(user)
        __video__(conf, obj)
    if etype == Type.Artist:
        __artist__(conf, obj)
    if etype == Type.Playlist:
        __playlist__(conf, obj)






#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   turkish.py
@Time    :   2020/08/20
@Author  :   shhade for hack
@Version :   1.0
@Contact :   
@Desc    :   
'''

class LangTurkish(object):
    SETTING = "AYARLAR"
    VALUE = "DEĞER"
    SETTING_DOWNLOAD_PATH = "Inenler dosyası"
    SETTING_ONLY_M4A = "mp4'ü m4a ya çevirmek"
    SETTING_ADD_EXPLICIT_TAG = "Açık etiketini ekle"
    SETTING_ADD_HYPHEN = "tire ekle"
    SETTING_ADD_YEAR = "Albüm-dosyasından önce yılı ekle"
    SETTING_USE_TRACK_NUM = "Kullanıcı takip numarasını ekle"
    SETTING_AUDIO_QUALITY = "Ses Kalitesi"
    SETTING_VIDEO_QUALITY = "Video Kalitesi"
    SETTING_CHECK_EXIST = "dosyanın olup olnadığına bak"
    SETTING_ARTIST_BEFORE_TITLE = "artist ismi şarkı-isminden önce"
    SETTING_ALBUMID_BEFORE_FOLDER = "Takip numarsı indirme dosyasından önce"
    SETTING_INCLUDE_EP = "ekle single&ep"
    SETTING_SAVE_COVERS = "kapak resmini kaydet"
    SETTING_LANGUAGE = "Dil"

    CHOICE = "SEÇENEK"
    FUNCTION = "FONKSİYON"
    CHOICE_ENTER = "Gir"
    CHOICE_ENTER_URLID = "Url/ID Gir:"
    CHOICE_EXIT = "Çık"
    CHOICE_LOGIN = "Giriş yap"
    CHOICE_SETTINGS = "Ayarlar"
    CHOICE_SET_ACCESS_TOKEN = "access token gir"
    CHOICE_DOWNLOAD_BY_URL = "id numarası veya web bağlantısıyla indir"

    PRINT_ERR = "[HATA]"
    PRINT_INFO = "[BILGI]"
    PRINT_SUCCESS = "[BAŞARILI]"

    PRINT_ENTER_CHOICE = "Seçeneği Gir:"
    PRINT_LATEST_VERSION = "Son Sürüm:"
    PRINT_USERNAME = "kullanıcı adı:"
    PRINT_PASSWORD = "şifre:"
    
    CHANGE_START_SETTINGS = "ayarları başlat('0'-dön,'1'-evet):"
    CHANGE_DOWNLOAD_PATH = "indirlenler dosyası('0' değişmesin):"
    CHANGE_AUDIO_QUALITY = "ses kalitesi('0'-Normal,'1'-Yüksek,'2'-HiFi,'3'-Master):"
    CHANGE_VIDEO_QUALITY = "Video kalitesi('0'-1080,'1'-720,'2'-480,'3'-360):"
    CHANGE_ONLYM4A = "mp4 ü m4a ya çevir('0'-hayır,'1'-Evet):"
    CHANGE_ADD_EXPLICIT_TAG = "Açık etiketi ekle('0'-Hayır,'1'-Evet):"
    CHANGE_ADD_HYPHEN = "boşluk yerine satır ekle('0'-Hayır,'1'-Evet):"
    CHANGE_ADD_YEAR = "Albüm dosyasına albümün çıkış yılını ekle('0'-Hayır,'1'-Evet):"
    CHANGE_USE_TRACK_NUM = "şarkı numarasını şarkı isminden önce ekle('0'-Hayır,'1'-Evet):"
    CHANGE_CHECK_EXIST = "indirmeden önce dosyanın olup olmadığın kontrol et('0'-Hayır,'1'-Evet):"
    CHANGE_ARTIST_BEFORE_TITLE = "artist adını şarkı isminden önce ekle('0'-Hayır,'1'-Evet):"
    CHANGE_INCLUDE_EP = "single şarkıları albümü indirirken ekle('0'-Hayır,'1'-Evet):"
    CHANGE_ALBUMID_BEFORE_FOLDER = "album numarasını dosya numarsından önce değiştir('0'-Hayır ,'1'-Evet):"
    CHANGE_SAVE_COVERS = "Kapak resmini kaydet('0'-Hayır ,'1'-Evet):"
    CHANGE_LANGUAGE = "Dil seç"

    MSG_INVAILD_ACCESSTOKEN = "Hatalı AccessToken! ."
    MSG_PATH_ERR = "Indirilen dosyası hatalı!"
    MSG_INPUT_ERR = "Giriş Hatalı!"


    MODEL_ALBUM_PROPERTY = "ALBUM-MÜLKİYETI"
    MODEL_TRACK_PROPERTY = "ŞARKI-MÜLKİYETI"
    MODEL_VIDEO_PROPERTY = "VIDEO-MÜLKİYETİ"
    MODEL_ARTIST_PROPERTY = "ARTIST-MÜLKİYETI"
    MODEL_PLAYLIST_PROPERTY = "OYNATMA LİSTESİ-MÜLKİYETİ"

    MODEL_TITLE = 'İsim'
    MODEL_TRACK_NUMBER = 'şarkı numarsı'
    MODEL_VIDEO_NUMBER = 'Video numarası'
    MODEL_RELEASE_DATE = 'Çıkış Yılı'
    MODEL_VERSION = 'Versiyonu'
    MODEL_EXPLICIT = 'Açık '
    MODEL_ALBUM = 'Album'
    MODEL_ID = 'Takip Numarası'
    MODEL_NAME = 'Isim'
    MODEL_TYPE = 'Tip'



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   language.py
@Time    :   2020/08/19
@Author  :   Yaronzz
@Version :   1.0
@Contact :   yaronhuang@foxmail.com
@Desc    :   
'''
from tidal_dl.lang.english import LangEnglish
from tidal_dl.lang.chinese import LangChinese
from tidal_dl.lang.turkish import LangTurkish
from tidal_dl.lang.italian import LangItalian
from tidal_dl.lang.czech import LangCzech
from tidal_dl.lang.arabic import LangArabic
from tidal_dl.lang.russian import LangRussian


def initLang(index):  # 初始化
    global LANG
    return setLang(index)

def setLang(index):
    global LANG
    if str(index) == '0':
        LANG = LangEnglish()
    elif str(index) == '1':
        LANG = LangChinese()
    elif str(index) == '2':
        LANG = LangTurkish()
    elif str(index) == '3':
        LANG = LangItalian()
    elif str(index) == '4':
        LANG = LangCzech()
    elif str(index) == '5':
        LANG = LangArabic()
    elif str(index) == '6':
        LANG = LangRussian()
    else:
        LANG = LangEnglish()
    return LANG

def getLang():
    global LANG
    return LANG

def getLangName(index):
    if str(index) == '0':
        return "English"
    if str(index) == '1':
        return "中文"
    if str(index) == '2':
        return "Turkish"
    if str(index) == '3':
        return "Italian"
    if str(index) == '4':
        return "Czech"
    if str(index) == '5':
        return "Arabic"
    if str(index) == '6':
        return "Russian"
    return "English"



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   arabic.py
@Time    :   2020/08/19
@Author  :   shhade for hack
@Version :   1.0
@Contact :   
@Desc    :   
'''

class LangArabic(object):
    SETTING = "الاعدادت"
    VALUE = "القيمة"
    SETTING_DOWNLOAD_PATH = "مجلد التحميل"
    SETTING_ONLY_M4A = "تحويل m4a الى mp4"
    SETTING_ADD_EXPLICIT_TAG = "اضافة توقيع الفنان"
    SETTING_ADD_HYPHEN = "اضافة سطر تحتي"
    SETTING_ADD_YEAR = "اضافة سنة الاصدار قبل نجلد التنزيل"
    SETTING_USE_TRACK_NUM = "اضف رقم التتبع الخاص بالاغنية"
    SETTING_AUDIO_QUALITY = "دقة الصوت"
    SETTING_VIDEO_QUALITY = "دقة الفديو"
    SETTING_CHECK_EXIST = "التاكد من وجود الملف قبل التنزيل"
    SETTING_ARTIST_BEFORE_TITLE = "اسم الفنان قبل اسم الاغنية"
    SETTING_ALBUMID_BEFORE_FOLDER = "رقم التتبع قبل مجلد التنزيل"
    SETTING_INCLUDE_EP = "اضافة single&ep"
    SETTING_SAVE_COVERS = "حفظ صورة الالبوم"
    SETTING_LANGUAGE = "اللغة"

    CHOICE = "خيار"
    FUNCTION = "وظيفة"
    CHOICE_ENTER = "ادخل"
    CHOICE_ENTER_URLID = "ادخل 'رابط/رقم تتبع':"
    CHOICE_EXIT = "اخرج"
    CHOICE_LOGIN = "تسجيل الدخول"
    CHOICE_SETTINGS = "الاعدادات"
    CHOICE_SET_ACCESS_TOKEN = "اعداد AccessToken"
    CHOICE_DOWNLOAD_BY_URL = "التحميل عبر الرابط او رقم التتبع"

    PRINT_ERR = "[خطأ]"
    PRINT_INFO = "[معلومة]"
    PRINT_SUCCESS = "[نجاح]"

    PRINT_ENTER_CHOICE = "ادخل الخيار:"
    PRINT_LATEST_VERSION = "آخر اصدر:"
    PRINT_USERNAME = "اسم المستخدم:"
    PRINT_PASSWORD = "رمز الدخول:"
    
    CHANGE_START_SETTINGS = "بدء الاعدادات('0'-Return,'1'-Yes):"
    CHANGE_DOWNLOAD_PATH = "مجلد التنزيل('0' not modify):"
    CHANGE_AUDIO_QUALITY = "دقة الصوت('0'-Normal,'1'-High,'2'-HiFi,'3'-Master):"
    CHANGE_VIDEO_QUALITY = "دقة الفديو('0'-1080,'1'-720,'2'-480,'3'-360):"
    CHANGE_ONLYM4A = "تحويل M4a الى mp4('0'-No,'1'-Yes):"
    CHANGE_ADD_EXPLICIT_TAG = "اضافة توقيع الفنان('0'-No,'1'-Yes):"
    CHANGE_ADD_HYPHEN = "استخدم السطور التحتية بدل الفراغات('0'-No,'1'-Yes):"
    CHANGE_ADD_YEAR = "اضافة سنة الاصدار الى مجلد الالبوم('0'-No,'1'-Yes):"
    CHANGE_USE_TRACK_NUM = "اضافة رقم الاغنية قبل اسمها('0'-No,'1'-Yes):"
    CHANGE_CHECK_EXIST = "التحقق من وجود الملف قبل التحميل('0'-No,'1'-Yes):"
    CHANGE_ARTIST_BEFORE_TITLE = "اضف اسم الفنان قبل اسم الاغنية('0'-No,'1'-Yes):"
    CHANGE_INCLUDE_EP = "اضافة مسطلحات فردي او ثنائي الى الغنية('0'-No,'1'-Yes):"
    CHANGE_ALBUMID_BEFORE_FOLDER = "اضف رقم التتبع قبل اسم مجلد الالبوم('0'-No,'1'-Yes):"
    CHANGE_SAVE_COVERS = "حفظ صورة الالبوم('0'-No,'1'-Yes):"
    CHANGE_LANGUAGE = "اختر لغة"

    MSG_INVAILD_ACCESSTOKEN = "خاطئة AccessToken! الرجاء الاعادة."
    MSG_PATH_ERR = "!مجلد التنزيل خاطئ"
    MSG_INPUT_ERR = "!ادخال خاطئ"


    MODEL_ALBUM_PROPERTY = "ملكية الالبوم"
    MODEL_TRACK_PROPERTY = "ملكية الاغنية"
    MODEL_VIDEO_PROPERTY = "ملكية الفديو"
    MODEL_ARTIST_PROPERTY = "ملكية الفنان"
    MODEL_PLAYLIST_PROPERTY = "ملكية قائمة الاغاني"

    MODEL_TITLE = 'الاسم'
    MODEL_TRACK_NUMBER = 'رقم الاغنية'
    MODEL_VIDEO_NUMBER = 'رقم الفديو'
    MODEL_RELEASE_DATE = 'عام الاصدار'
    MODEL_VERSION = 'الاصدار'
    MODEL_EXPLICIT = 'توقيع الفنان'
    MODEL_ALBUM = 'الالبوم'
    MODEL_ID = 'رقم التتبع'
    MODEL_NAME = 'الاسم'
    MODEL_TYPE = 'النوع'



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   __init__.py
@Time    :   2020/08/15
@Author  :   Yaronzz
@Version :   1.0
@Contact :   yaronhuang@foxmail.com
@Desc    :   
'''
import os
import requests
import prettytable

from aigpy.stringHelper import isNull
from aigpy.pathHelper import mkdirs
from aigpy.pipHelper import getLastVersion
from aigpy.versionHelper import cmpVersion

from tidal_dl.tidal import TidalAPI
from tidal_dl.settings import Settings, UserSettings
from tidal_dl.printf import Printf, VERSION
from tidal_dl.download import start
from tidal_dl.enum import AudioQuality, VideoQuality
from tidal_dl.lang.language import getLang, setLang, initLang

API = TidalAPI()
USER = UserSettings.read()
CONF = Settings.read()
TOKEN1, TOKEN2 = API.getToken()
LANG = initLang(CONF.language)

def login(username="", password=""):
    while True:
        if isNull(username) or isNull(password):
            print("---------------" + LANG.CHOICE_LOGIN + "-----------------")
            username = Printf.enter(LANG.PRINT_USERNAME)
            password = Printf.enter(LANG.PRINT_PASSWORD)
        msg, check = API.login(username, password, TOKEN1)
        if check == False:
            Printf.err(msg)
            username = ""
            password = ""
            continue
        api2 = TidalAPI()
        msg, check = api2.login(username, password, TOKEN2)
        break
    
    USER.username = username
    USER.password = password
    USER.userid = API.key.userId
    USER.countryCode = API.key.countryCode
    USER.sessionid1 = API.key.sessionId
    USER.sessionid2 = api2.key.sessionId
    UserSettings.save(USER)



def setAccessToken():
    while True:
        print("-------------AccessToken---------------")
        token = Printf.enter("accessToken('0' go back):")
        if token == '0':
            return
        msg, check = API.loginByAccessToken(token, USER.userid)
        if check == False:
            Printf.err(msg)
            continue
        break

    USER.assesstoken = token
    UserSettings.save(USER)



def checkLogin():
    if not isNull(USER.assesstoken):
        mag, check = API.loginByAccessToken(USER.assesstoken)
        if check == False:
            Printf.err(LANG.MSG_INVAILD_ACCESSTOKEN)
    if not isNull(USER.sessionid1) and not API.isValidSessionID(USER.userid, USER.sessionid1):
        USER.sessionid1 = ""
    if not isNull(USER.sessionid2) and API.isValidSessionID(USER.userid, USER.sessionid2):
        USER.sessionid2 = ""
    if isNull(USER.sessionid1) or isNull(USER.sessionid2):
        login(USER.username, USER.password)



def changeSettings():
    global LANG
    
    Printf.settings(CONF)
    choice = Printf.enter(LANG.CHANGE_START_SETTINGS)
    if choice == '0':
        return

    while True:
        choice = Printf.enter(LANG.CHANGE_DOWNLOAD_PATH)
        if choice == '0':
            choice = CONF.downloadPath
        elif not os.path.isdir(choice):
            if not mkdirs(choice):
                Printf.err(LANG.MSG_PATH_ERR)
                continue
        CONF.downloadPath = choice
        break
    while True:
        choice = Printf.enter(LANG.CHANGE_AUDIO_QUALITY)
        if choice != '1' and choice != '2' and choice != '3' and choice != '0':
            Printf.err(LANG.MSG_INPUT_ERR)
            continue
        if choice == '0':
            CONF.audioQuality = AudioQuality.Normal
        if choice == '1':
            CONF.audioQuality = AudioQuality.High
        if choice == '2':
            CONF.audioQuality = AudioQuality.HiFi
        if choice == '3':
            CONF.audioQuality = AudioQuality.Master
        break
    while True:
        choice = Printf.enter(LANG.CHANGE_VIDEO_QUALITY)
        if choice != '1' and choice != '2' and choice != '3' and choice != '0':
            Printf.err(LANG.MSG_INPUT_ERR)
            continue
        if choice == '0':
            CONF.videoQuality = VideoQuality.P1080
        if choice == '1':
            CONF.videoQuality = VideoQuality.P720
        if choice == '2':
            CONF.videoQuality = VideoQuality.P480
        if choice == '3':
            CONF.videoQuality = VideoQuality.P360
        break
    CONF.onlyM4a = Printf.enter(LANG.CHANGE_ONLYM4A) == '1'
    CONF.addExplicitTag = Printf.enter(LANG.CHANGE_ADD_EXPLICIT_TAG) == '1'
    CONF.addHyphen = Printf.enter(LANG.CHANGE_ADD_HYPHEN) == '1'
    CONF.addYear = Printf.enter(LANG.CHANGE_ADD_YEAR) == '1'
    CONF.useTrackNumber = Printf.enter(LANG.CHANGE_USE_TRACK_NUM) == '1'
    CONF.checkExist = Printf.enter(LANG.CHANGE_CHECK_EXIST) == '1'
    CONF.artistBeforeTitle = Printf.enter(LANG.CHANGE_ARTIST_BEFORE_TITLE) == '1'
    CONF.includeEP = Printf.enter(LANG.CHANGE_INCLUDE_EP) == '1'
    CONF.addAlbumIDBeforeFolder = Printf.enter(LANG.CHANGE_ALBUMID_BEFORE_FOLDER) == '1'
    CONF.saveCovers = Printf.enter(LANG.CHANGE_SAVE_COVERS) == '1'
    CONF.language = Printf.enter(LANG.CHANGE_LANGUAGE +
                                 "('0'-English,'1'-中文,'2'-Turkish,'3'-Italiano,'4'-Czech,'5'-Arabic,'6'-Russian):")

    LANG = setLang(CONF.language)
    Settings.save(CONF)


def main():
    Printf.logo()
    Printf.settings(CONF)
    # checkLogin()

    onlineVer = getLastVersion('tidal-dl')
    if not isNull(onlineVer):
        icmp = cmpVersion(onlineVer, VERSION)
        if icmp > 0:
            Printf.info(LANG.PRINT_LATEST_VERSION + ' ' + onlineVer)

    while True:
        Printf.choices()
        choice = Printf.enter(LANG.PRINT_ENTER_CHOICE)
        if choice == "0":
            return
        elif choice == "1":
            login()
        elif choice == "2":
            changeSettings()
        elif choice == "3":
            setAccessToken()
        else:
            start(USER, CONF, choice)

if __name__ == "__main__":
    main()



